/*
 * Created on 28-Jul-2003
 */
package protocols.perfectFailureDetector;

import utils.ProcessSet;
import utils.SampleProcess;
import events.Crash;
import events.ProcessInitEvent;
import appia.AppiaEventException;
import appia.Channel;
import appia.Direction;
import appia.Event;
import appia.Layer;
import appia.Session;
import appia.events.channel.ChannelInit;
import appia.protocols.common.InetWithPort;
import appia.protocols.tcpcomplete.TcpUndeliveredEvent;

/**
 * @author nuno
 */
public class PerfectFailureDetectorSession extends Session {

	private Channel channel;
	private ProcessSet processes;
	private boolean started;

	/**
	 * Constructor of the Session.
	 * @param layer parent layer.
	 */
	public PerfectFailureDetectorSession(Layer layer) {
		super(layer);
		started = false;
	}

	/**
	 * handle incoming events.
	 * @see appia.Session#handle(appia.Event)
	 */
	public void handle(Event event) {
		if (event instanceof TcpUndeliveredEvent)
			notifyCrash((TcpUndeliveredEvent) event);
		else if (event instanceof ChannelInit)
			handleChannelInit((ChannelInit) event);
		else if (event instanceof ProcessInitEvent)
			handleProcessInit((ProcessInitEvent) event);
		else if(event instanceof PFDStartEvent)
			handlePFDStart((PFDStartEvent) event);
	}

	/**
	 * @param init
	 */
	private void handleChannelInit(ChannelInit init) {
		channel = init.getChannel();
		try {
			init.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param event
	 */
	private void handleProcessInit(ProcessInitEvent event) {
		processes = event.getProcessSet();
		try {
			event.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param event
	 */
	private void handlePFDStart(PFDStartEvent event) {
		started = true;
		try {
			event.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
		try {
			CreateChannelsEvent createChannels =
				new CreateChannelsEvent(
					channel,
					Direction.DOWN,
					this);
			createChannels.go();
		} catch (AppiaEventException e1) {
			e1.printStackTrace();
		}			
	}

	/**
	 * When this protocol receives a TcpUndelivered Event from a process 
	 * that was correct, it creates a Crash event and sends it to above protocols.
	 * @param event
	 */
	private void notifyCrash(TcpUndeliveredEvent event) {
		if(started){
			SampleProcess p = processes.getProcess((InetWithPort) event.who);
			if (p.isCorrect()) {
				p.setCorrect(false);
				try {
					Crash crash =
					new Crash(
							channel,
							Direction.UP,
							this,
							p.getProcessNumber());
					crash.go();
				} catch (AppiaEventException e) {
					e.printStackTrace();
				}
			}
		}
	}
}
