/*
 * PBLayer.java
 * Created on 22-Sep-2003, 17:09:17
 */
package protocols.probabilisticBroadcast;

import events.ProcessInitEvent;
import appia.Layer;
import appia.Session;
import appia.events.SendableEvent;
import appia.events.channel.ChannelClose;
import appia.events.channel.ChannelInit;

/**
 * Class PBLayer. Layer of the Probabilistic Broadcast protocol. 
 * @author nuno
 */
public class PBLayer extends Layer {

	private int fanout;
	private int maxRounds;

	public PBLayer(){
		/* events that the protocol will create */
		evProvide = new Class[0];
	
		/* events that the protocol require to work.
		 * This is a subset of the accepted events */
		evRequire = new Class[3];
		evRequire[0] = SendableEvent.class;
		evRequire[1] = ChannelInit.class;
		evRequire[2] = ProcessInitEvent.class; 
	
		/* events that the protocol will accept */
		evAccept = new Class[4];
		evAccept[0] = SendableEvent.class;
		evAccept[1] = ChannelInit.class;
		evAccept[2] = ChannelClose.class;
		evAccept[3] = ProcessInitEvent.class;		
	}

	/**
	 * Creates a new session to this protocol.
	 * @see appia.Layer#createSession()
	 */
	public Session createSession() {
		return new PBSession(this);
	}


	/**
	 * Sets the values that will be passed to the Session.
	 * @param f fanout
	 * @param r number of rounds
	 */
	public void initValues(int f, int r){
		fanout = f;
		maxRounds = r;
	}

	/**
	 * Gets the fanout.
	 * @return
	 */
	public int getFanout() {
		return fanout;
	}

	/**
	 * gets the number of rounds.
	 * @return
	 */
	public int getMaxRounds() {
		return maxRounds;
	}
}
