/*
 * Created on 21-Jul-2003
 */
package protocols.sampleAppl;

import protocols.perfectFailureDetector.PFDStartEvent;

import utils.ProcessSet;
import events.ProcessInitEvent;
import events.SampleSendableEvent;
import appia.AppiaEventException;
import appia.Channel;
import appia.Direction;
import appia.Event;
import appia.Layer;
import appia.Session;
import appia.events.channel.ChannelClose;
import appia.events.channel.ChannelInit;
import appia.message.ExtendedMessage;
import appia.protocols.udpsimple.RegisterSocketEvent;


/**
 * @author nuno
 */
public class SampleApplSession extends Session {

	Channel channel;
	private ProcessSet processes;
	private SampleApplReader reader;

	/**
	 * @param layer
	 */
	public SampleApplSession(Layer layer) {
		super(layer);
	}

	/**
	 * @param string
	 * @param self
	 */
	public void init(ProcessSet processes) {
		this.processes = processes;
	}

	public void handle(Event event){
		//System.out.println("Received event: "+event.getClass().getName());
		if(event instanceof SampleSendableEvent)
			handleSampleSendableEvent((SampleSendableEvent) event);
		else if(event instanceof ChannelInit)
			handleChannelInit((ChannelInit)event);
		else if(event instanceof ChannelClose)
			handleChannelClose((ChannelClose)event);
		else if(event instanceof RegisterSocketEvent)
			handleRegisterSocket((RegisterSocketEvent)event);
	}

	/**
	 * @param event
	 */
	private void handleRegisterSocket(RegisterSocketEvent event) {
		if(event.error){
			System.out.println("Address already in use!");
			System.exit(2);
		}
	}

	/**
	 * @param init
	 */
	private void handleChannelInit(ChannelInit init) {
		try {
			init.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
		channel = init.getChannel();
				
		try {
			// sends this event to open a socket in the layer that is used has perfect point to point
			// channels or unreliable point to point channels.
			int port = processes.getSelfProcess().getInetWithPort().port;
			RegisterSocketEvent rse = new RegisterSocketEvent(channel,Direction.DOWN,this,port);
			rse.go();
			ProcessInitEvent processInit = new ProcessInitEvent(channel,Direction.DOWN,this);
			processInit.setProcessSet(processes);
			processInit.go();
		} catch (AppiaEventException e1) {
			e1.printStackTrace();
		}
		System.out.println("Channel is open.");
		// starts the thread that reads from the keyboard.
		reader = new SampleApplReader(this);
		reader.start();
	}

	/**
	 * @param close
	 */
	private void handleChannelClose(ChannelClose close) {
		channel = null;
		System.out.println("Channel is closed.");
	}

	/**
	 * @param event
	 */
	private void handleSampleSendableEvent(SampleSendableEvent event) {
		if(event.getDir() == Direction.DOWN)
			handleOutgoingEvent(event);
		else
			handleIncomingEvent(event);
	}

	/**
	 * @param event
	 */
	private void handleIncomingEvent(SampleSendableEvent event) {
		String message = ((ExtendedMessage)event.getMessage()).popString();
		System.out.println("Received event with message: "+message);
	}

	/**
	 * @param event
	 */
	private void handleOutgoingEvent(SampleSendableEvent event) {
		String command = event.getCommand();
		if("bcast".equals(command))
			handleBCast(event);
		else if("startpfd".equals(command))
			handleStartPFD(event);
		else if("help".equals(command))
			printHelp();
		else{
			System.out.println("Invalid command: "+command);
			printHelp();
		}
	}

	/**
	 * @param event
	 */
	private void handleBCast(SampleSendableEvent event) {
		try {
			event.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param event
	 */
	private void handleStartPFD(SampleSendableEvent event) {
		try {
			PFDStartEvent pfdStart = new PFDStartEvent(channel,Direction.DOWN,this);
			pfdStart.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	private void printHelp() {
		System.out.println("Available commands:\n"+
				"startpfd - starts the Perfect Failure detector (when applies\n"+
				"bcast <msg> - Broadcast the message \"msg\"\n"+
				"help - Print this help information.");
	}

}

